from __future__ import absolute_import
from optparse import make_option
from subprocess import CalledProcessError
import logging
import os
import time

from backupdb.utils.commands import BaseBackupDbCommand, do_postgresql_backup
from backupdb.utils.exceptions import BackupError
from backupdb.utils.log import section, SectionError, SectionWarning
from backupdb.utils.settings import BACKUP_DIR, BACKUP_CONFIG

logger = logging.getLogger(__name__)


class Command(BaseBackupDbCommand):
    help = 'Backs up each database in settings.DATABASES.'

    option_list = BaseBackupDbCommand.option_list + (
        make_option(
            '--backup-name',
            help=(
                'Specify a name for the backup.  Defaults to the current '
                'timestamp.  Example: `--backup-name=test` will create backup '
                'files that look like "default-test.pgsql.gz".'
            ),
        ),
        make_option(
            '--pg-dump-options',
            help=(
                'For postgres backups, a list of additional options that will '
                'be passed through to the pg_dump utility.  Example: '
                '`--pg-dump-options="--inserts --no-owner"`'
            ),
        ),
        make_option(
            '--show-output',
            action='store_true',
            default=False,
            help=(
                'Display the output of stderr and stdout (apart from data which '
                'is piped from one process to another) for processes that are '
                'run while backing up databases.  These are command-line '
                'programs such as `psql` or `mysql`.  This can be useful for '
                'understanding how backups are failing in the case that they '
                'are.'
            ),
        ),
    )

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)

        from django.conf import settings

        current_time = time.strftime('%F-%s')
        backup_name = options['backup_name'] or current_time
        show_output = options['show_output']

        # Ensure backup dir present
        if not os.path.exists(BACKUP_DIR):
            os.makedirs(BACKUP_DIR)

        # Loop through databases
        for db_name, db_config in settings.DATABASES.items():
            with section("Backing up '{0}'...".format(db_name)):
                # Get backup config for this engine type
                engine = db_config['ENGINE']
                backup_config = BACKUP_CONFIG.get(engine)
                if not backup_config:
                    raise SectionWarning("Backup for '{0}' engine not implemented".format(engine))

                # Get backup file name
                backup_base_name = '{db_name}-{backup_name}.{backup_extension}.gz'.format(
                    db_name=db_name,
                    backup_name=backup_name,
                    backup_extension=backup_config['backup_extension'],
                )
                backup_file = os.path.join(BACKUP_DIR, backup_base_name)

                # Find backup command and get kwargs
                backup_func = backup_config['backup_func']
                backup_kwargs = {
                    'backup_file': backup_file,
                    'db_config': db_config,
                    'show_output': show_output,
                }
                if backup_func is do_postgresql_backup:
                    backup_kwargs['pg_dump_options'] = options['pg_dump_options']

                # Run backup command
                try:
                    backup_func(**backup_kwargs)
                    logger.info("Backup of '{db_name}' saved in '{backup_file}'".format(
                        db_name=db_name,
                        backup_file=backup_file))
                except (BackupError, CalledProcessError) as e:
                    raise SectionError(e)



from __future__ import absolute_import
from optparse import make_option
from subprocess import CalledProcessError
import logging
import os

from django.core.management.base import CommandError
from django.conf import settings
from django.db import close_connection

from backupdb.utils.commands import BaseBackupDbCommand
from backupdb.utils.exceptions import RestoreError
from backupdb.utils.files import get_latest_timestamped_file
from backupdb.utils.log import section, SectionError, SectionWarning
from backupdb.utils.settings import BACKUP_DIR, BACKUP_CONFIG

logger = logging.getLogger(__name__)


class Command(BaseBackupDbCommand):
    help = 'Restores each database in settings.DATABASES from latest db backup.'

    option_list = BaseBackupDbCommand.option_list + (
        make_option(
            '--backup-name',
            help=(
                'Name of backup to restore from.  Example: '
                '`--backup-name=mybackup` will restore any backups that look '
                'like "default-mybackup.pgsql.gz".  Defaults to latest '
                'timestamped backup name.'
            ),
        ),
        make_option(
            '--drop-tables',
            action='store_true',
            default=False,
            help=(
                '** USE WITH CAUTION ** Drop all tables in databases before '
                'restoring them. The SQL dumps which are created by backupdb '
                'have drop statements so, usually, this option is not '
                'necessary.'
            ),
        ),
        make_option(
            '--show-output',
            action='store_true',
            default=False,
            help=(
                'Display the output of stderr and stdout (apart from data which '
                'is piped from one process to another) for processes that are '
                'run while restoring databases.  These are command-line '
                'programs such as `psql` or `mysql`.  This can be useful for '
                'understanding how restoration is failing in the case that it '
                'is.'
            ),
        ),
    )

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)

        # Django is querying django_content_types in a hanging transaction
        # Because of this psql can't drop django_content_types and just hangs
        close_connection()

        # Ensure backup dir present
        if not os.path.exists(BACKUP_DIR):
            raise CommandError("Backup dir '{0}' does not exist!".format(BACKUP_DIR))

        backup_name = options['backup_name']
        drop_tables = options['drop_tables']
        show_output = options['show_output']

        # Loop through databases
        for db_name, db_config in settings.DATABASES.items():
            with section("Restoring '{0}'...".format(db_name)):
                # Get backup config for this engine type
                engine = db_config['ENGINE']
                backup_config = BACKUP_CONFIG.get(engine)
                if not backup_config:
                    raise SectionWarning("Restore for '{0}' engine not implemented".format(engine))

                # Get backup file name
                backup_extension = backup_config['backup_extension']
                if backup_name:
                    backup_file = '{dir}/{db_name}-{backup_name}.{ext}.gz'.format(
                        dir=BACKUP_DIR,
                        db_name=db_name,
                        backup_name=backup_name,
                        ext=backup_extension,
                    )
                else:
                    try:
                        backup_file = get_latest_timestamped_file(backup_extension)
                    except RestoreError as e:
                        raise SectionError(e)

                # Find restore command and get kwargs
                restore_func = backup_config['restore_func']
                restore_kwargs = {
                    'backup_file': backup_file,
                    'db_config': db_config,
                    'drop_tables': drop_tables,
                    'show_output': show_output,
                }

                # Run restore command
                try:
                    restore_func(**restore_kwargs)
                    logger.info("Restored '{db_name}' from '{backup_file}'".format(
                        db_name=db_name,
                        backup_file=backup_file))
                except (RestoreError, CalledProcessError) as e:
                    raise SectionError(e)



from .commands import (
    do_mysql_backup,
    do_mysql_restore,
    do_postgresql_backup,
    do_postgresql_restore,
    do_sqlite_backup,
    do_sqlite_restore,
)
from django.conf import settings


DEFAULT_BACKUP_DIR = 'backups'
BACKUP_DIR = getattr(settings, 'BACKUPDB_DIRECTORY', DEFAULT_BACKUP_DIR)
BACKUP_TIMESTAMP_PATTERN = '*-[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]-[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]'
BACKUP_CONFIG = {
    'django.db.backends.mysql': {
        'backup_extension': 'mysql',
        'backup_func': do_mysql_backup,
        'restore_func': do_mysql_restore,
    },
    'django.db.backends.postgresql_psycopg2': {
        'backup_extension': 'pgsql',
        'backup_func': do_postgresql_backup,
        'restore_func': do_postgresql_restore,
    },
    'django.contrib.gis.db.backends.postgis': {
        'backup_extension': 'pgsql',
        'backup_func': do_postgresql_backup,
        'restore_func': do_postgresql_restore,
    },
    'django.db.backends.sqlite3': {
        'backup_extension': 'sqlite',
        'backup_func': do_sqlite_backup,
        'restore_func': do_sqlite_restore,
    },
}



import unittest

from backupdb.utils.log import bar


class BarTestCase(unittest.TestCase):
    def test_it_should_return_a_string_with_text_centered_in_a_bar(self):
        self.assertEqual(
            bar('test', width=70),
            '================================ test ================================',
        )

    def test_it_should_return_a_bar_string_with_the_specified_width(self):
        test_bar1 = r'=========================== test ==========================='
        test_bar2 = r'//========================= test =========================\\'
        test_bar3 = r'\\========================= test =========================//'

        self.assertEqual(len(test_bar1), 60)
        self.assertEqual(len(test_bar2), 60)
        self.assertEqual(len(test_bar3), 60)

        self.assertEqual(bar('test', width=60), test_bar1)
        self.assertEqual(bar('test', width=60, position='top'), test_bar2)
        self.assertEqual(bar('test', width=60, position='bottom'), test_bar3)

    def test_it_should_work_even_if_the_given_width_is_less_than_the_message_length(self):
        self.assertEqual(bar('test', width=0), '== test ==')
        self.assertEqual(bar('test', width=0, position='top'), r'// test \\')
        self.assertEqual(bar('test', width=0, position='bottom'), r'\\ test //')

    def test_it_should_render_a_top_bottom_or_plain_bar_depending_on_the_position_argument(self):
        test_bar1 = r'================================ test ================================'
        test_bar2 = r'//============================== test ==============================\\'
        test_bar3 = r'\\============================== test ==============================//'

        self.assertEqual(bar('test', width=70), test_bar1)
        self.assertEqual(bar('test', width=70, position='top'), test_bar2)
        self.assertEqual(bar('test', width=70, position='bottom'), test_bar3)

    def test_it_should_allow_the_message_to_be_blank(self):
        test_bar1 = r'======================================================================'
        test_bar2 = r'//==================================================================\\'
        test_bar3 = r'\\==================================================================//'

        self.assertEqual(bar(width=70), test_bar1)
        self.assertEqual(bar(width=70, position='top'), test_bar2)
        self.assertEqual(bar(width=70, position='bottom'), test_bar3)



import glob

from .exceptions import RestoreError
from .settings import BACKUP_DIR, BACKUP_TIMESTAMP_PATTERN


def get_latest_timestamped_file(ext, dir=BACKUP_DIR, pattern=BACKUP_TIMESTAMP_PATTERN):
    """
    Gets the latest timestamped backup file name with the given database type
    extension.
    """
    pattern = '{dir}/{pattern}.{ext}.gz'.format(
        dir=dir,
        pattern=pattern,
        ext=ext,
    )

    l = glob.glob(pattern)
    l.sort()
    l.reverse()

    if not l:
        raise RestoreError("No backups found matching '{0}' pattern".format(pattern))

    return l[0]



import os
import unittest


class FileSystemScratchTestCase(unittest.TestCase):
    SCRATCH_DIR = 'unit_tests_scratch'

    @classmethod
    def get_path(cls, file):
        return os.path.join(cls.SCRATCH_DIR, file)

    @classmethod
    def clear_scratch_dir(cls):
        """
        Deletes all scratch files in the tests scratch directory.
        """
        for file in os.listdir(cls.SCRATCH_DIR):
            if file != '.gitkeep':
                os.remove(cls.get_path(file))

    def setUp(self):
        self.clear_scratch_dir()

    def tearDown(self):
        self.clear_scratch_dir()

    def assertFileExists(self, file):
        self.assertTrue(os.path.exists(self.get_path(file)))

    def assertFileHasLength(self, file, length):
        with open(self.get_path(file)) as f:
            content = f.read()
            self.assertEqual(len(content), length)

    def assertFileHasContent(self, file, expected_content):
        with open(self.get_path(file)) as f:
            actual_content = f.read()
            self.assertEqual(actual_content, expected_content)

    def assertInFile(self, file, expected_content):
        with open(self.get_path(file)) as f:
            actual_content = f.read()
            self.assertTrue(expected_content in actual_content)



import unittest

from . import commands
from . import files
from . import log
from . import processes


loader = unittest.TestLoader()

commands_tests = loader.loadTestsFromModule(commands)
files_tests = loader.loadTestsFromModule(files)
log_tests = loader.loadTestsFromModule(log)
processes_tests = loader.loadTestsFromModule(processes)

all_tests = unittest.TestSuite([
    commands_tests,
    files_tests,
    log_tests,
    processes_tests,
])



import contextlib
import logging

logger = logging.getLogger(__name__)


def bar(msg='', width=40, position=None):
    r"""
    Returns a string with text centered in a bar caption.

    Examples:
    >>> bar('test', width=10)
    '== test =='
    >>> bar(width=10)
    '=========='
    >>> bar('Richard Dean Anderson is...', position='top', width=50)
    '//========= Richard Dean Anderson is... ========\\\\'
    >>> bar('...MacGyver', position='bottom', width=50)
    '\\\\================= ...MacGyver ================//'
    """
    if position == 'top':
        start_bar = '//'
        end_bar = r'\\'
    elif position == 'bottom':
        start_bar = r'\\'
        end_bar = '//'
    else:
        start_bar = end_bar = '=='

    if msg:
        msg = ' ' + msg + ' '

    width -= 4

    return start_bar + msg.center(width, '=') + end_bar


class SectionError(Exception):
    pass


class SectionWarning(Exception):
    pass


@contextlib.contextmanager
def section(msg):
    """
    Context manager that prints a top bar to stderr upon entering and a bottom
    bar upon exiting.  The caption of the top bar is specified by `msg`.  The
    caption of the bottom bar is '...done!' if the context manager exits
    successfully.  If a SectionError or SectionWarning is raised inside of the
    context manager, SectionError.message or SectionWarning.message is passed
    to logging.error or logging.warning respectively and the bottom bar caption
    becomes '...skipped.'.
    """
    logger.info(bar(msg, position='top'))
    try:
        yield
    except SectionError as e:
        logger.error(e.message)
        logger.info(bar('...skipped.', position='bottom'))
    except SectionWarning as e:
        logger.warning(e.message)
        logger.info(bar('...skipped.', position='bottom'))
    else:
        logger.info(bar('...done!', position='bottom'))



from subprocess import Popen, PIPE, CalledProcessError
import logging
import os
import shutil

logger = logging.getLogger(__name__)


def extend_env(extra_env):
    """
    Copies and extends the current environment with the values present in
    `extra_env`.
    """
    env = os.environ.copy()
    env.update(extra_env)
    return env


def get_env_str(env):
    """
    Gets a string representation of a dict as though it contained environment
    variable values.
    """
    return ' '.join("{0}='{1}'".format(k, v) for k, v in env.items())


def pipe_commands(cmds, extra_env=None, show_stderr=False, show_last_stdout=False):
    """
    Executes the list of commands piping each one into the next.
    """
    env = extend_env(extra_env) if extra_env else None
    env_str = (get_env_str(extra_env) + ' ') if extra_env else ''
    cmd_strs = [env_str + ' '.join(cmd) for cmd in cmds]

    logger.info('Running `{0}`'.format(' | '.join(cmd_strs)))

    with open('/dev/null', 'w') as NULL:
        # Start processes
        processes = []
        last_i = len(cmds) - 1
        for i, (cmd_str, cmd) in enumerate(zip(cmd_strs, cmds)):
            if i == last_i:
                p_stdout = None if show_last_stdout else NULL
            else:
                p_stdout = PIPE
            p_stdin = processes[-1][1].stdout if processes else None
            p_stderr = None if show_stderr else NULL

            p = Popen(cmd, env=env, stdout=p_stdout, stdin=p_stdin, stderr=p_stderr)
            processes.append((cmd_str, p))

        # Close processes
        error = False
        for cmd_str, p in processes:
            if p.stdout:
                p.stdout.close()
            if p.wait() != 0:
                error = True
        if error:
            raise CalledProcessError(cmd=cmd_str, returncode=p.returncode)


def pipe_commands_to_file(cmds, path, extra_env=None, show_stderr=False):
    """
    Executes the list of commands piping each one into the next and writing
    stdout of the last process into a file at the given path.
    """
    env = extend_env(extra_env) if extra_env else None
    env_str = (get_env_str(extra_env) + ' ') if extra_env else ''
    cmd_strs = [env_str + ' '.join(cmd) for cmd in cmds]

    logger.info('Saving output of `{0}`'.format(' | '.join(cmd_strs)))

    with open('/dev/null', 'w') as NULL:
        # Start processes
        processes = []
        for cmd_str, cmd in zip(cmd_strs, cmds):
            p_stdin = processes[-1][1].stdout if processes else None
            p_stderr = None if show_stderr else NULL

            p = Popen(cmd, env=env, stdout=PIPE, stdin=p_stdin, stderr=p_stderr)
            processes.append((cmd_str, p))

        p_last = processes[-1][1]

        with open(path, 'wb') as f:
            shutil.copyfileobj(p_last.stdout, f)

            # Close processes
            error = False
            for cmd_str, p in processes:
                p.stdout.close()
                if p.wait() != 0:
                    error = True
            if error:
                raise CalledProcessError(cmd=cmd_str, returncode=p.returncode)



from mock import call, patch
import unittest

from backupdb.utils.commands import (
    PG_DROP_SQL,
    get_mysql_args,
    get_postgresql_args,
    get_postgresql_env,
    do_mysql_backup,
    do_postgresql_backup,
    do_sqlite_backup,
    do_mysql_restore,
    do_postgresql_restore,
    do_sqlite_restore,
)
from backupdb.utils.exceptions import RestoreError


DB_CONFIG = {
    'NAME': 'test_db',
    'USER': 'test_user',
    'PASSWORD': 'test_password',
    'HOST': 'test_host',
    'PORT': 12345,
}


def make_db_config(*keys):
    new_dict = {}
    for k in keys:
        new_dict[k] = DB_CONFIG[k]
    return new_dict


class MockOsPathExists(object):
    """
    Used as a mock object for os.path.exists.
    """
    def __init__(self, *args, **kwargs):
        pass

    def __call__(self, *args, **kwargs):
        return True


class PatchPipeCommandsTestCase(unittest.TestCase):
    """
    Used for testing of pipe_commands and pipe_commands_to_file.
    """
    def setUp(self):
        self.pipe_commands_patcher = patch('backupdb.utils.commands.pipe_commands')
        self.pipe_commands_to_file_patcher = patch('backupdb.utils.commands.pipe_commands_to_file')
        self.os_patcher = patch('os.path.exists', new_callable=MockOsPathExists)

        self.mock_pipe_commands = self.pipe_commands_patcher.start()
        self.mock_pipe_commands_to_file = self.pipe_commands_to_file_patcher.start()
        self.mock_os = self.os_patcher.start()

    def tearDown(self):
        self.pipe_commands_patcher.stop()
        self.pipe_commands_to_file_patcher.stop()
        self.os_patcher.stop()

    def assertPipeCommandsCallsEqual(self, *args):
        self.assertEqual(self.mock_pipe_commands.call_args_list, list(args))

    def assertPipeCommandsToFileCallsEqual(self, *args):
        self.assertEqual(self.mock_pipe_commands_to_file.call_args_list, list(args))


class RequireBackupExistsTestCase(unittest.TestCase):
    def test_it_raises_an_exception_when_the_path_in_backup_file_arg_doesnt_exist(self):
        self.assertRaises(RestoreError, do_mysql_restore, backup_file='i_dont_exist', db_config={})
        self.assertRaises(RestoreError, do_postgresql_restore, backup_file='i_dont_exist', db_config={})
        self.assertRaises(RestoreError, do_sqlite_restore, backup_file='i_dont_exist', db_config={})


class GetMysqlArgsTestCase(unittest.TestCase):
    def test_it_builds_the_correct_args(self):
        self.assertEqual(
            get_mysql_args(make_db_config('NAME', 'USER')),
            [
                '--user=test_user',
                'test_db',
            ],
        )
        self.assertEqual(
            get_mysql_args(make_db_config('NAME', 'USER', 'PASSWORD')),
            [
                '--user=test_user',
                '--password=test_password',
                'test_db',
            ],
        )
        self.assertEqual(
            get_mysql_args(make_db_config('NAME', 'USER', 'PASSWORD', 'HOST')),
            [
                '--user=test_user',
                '--password=test_password',
                '--host=test_host',
                'test_db',
            ],
        )
        self.assertEqual(
            get_mysql_args(make_db_config('NAME', 'USER', 'PASSWORD', 'HOST', 'PORT')),
            [
                '--user=test_user',
                '--password=test_password',
                '--host=test_host',
                '--port=12345',
                'test_db',
            ],
        )


class GetPostgresqlArgsTestCase(unittest.TestCase):
    def test_it_builds_the_correct_args(self):
        self.assertEqual(
            get_postgresql_args(make_db_config('NAME', 'USER')),
            [
                '--username=test_user',
                'test_db',
            ],
        )
        self.assertEqual(
            get_postgresql_args(make_db_config('NAME', 'USER', 'HOST')),
            [
                '--username=test_user',
                '--host=test_host',
                'test_db',
            ],
        )
        self.assertEqual(
            get_postgresql_args(make_db_config('NAME', 'USER', 'HOST', 'PORT')),
            [
                '--username=test_user',
                '--host=test_host',
                '--port=12345',
                'test_db',
            ],
        )

    def test_it_correctly_includes_extra_args(self):
        self.assertEqual(
            get_postgresql_args(
                make_db_config('NAME', 'USER'),
                extra_args='--no-owner --no-privileges',
            ),
            [
                '--username=test_user',
                '--no-owner',
                '--no-privileges',
                'test_db',
            ],
        )


class GetPostgresqlEnvTestCase(unittest.TestCase):
    def test_it_builds_the_correct_env_dict(self):
        self.assertTrue(
            get_postgresql_env(make_db_config('USER', 'NAME')) is None,
        )
        self.assertEqual(
            get_postgresql_env(make_db_config('USER', 'NAME', 'PASSWORD')),
            {'PGPASSWORD': 'test_password'},
        )


class DoMysqlBackupTestCase(PatchPipeCommandsTestCase):
    def test_it_makes_correct_calls_to_processes_api(self):
        do_mysql_backup('test.mysql.gz', DB_CONFIG)

        self.assertPipeCommandsToFileCallsEqual(call(
            [
                [
                    'mysqldump',
                    '--user=test_user',
                    '--password=test_password',
                    '--host=test_host',
                    '--port=12345',
                    'test_db',
                ],
                ['gzip'],
            ],
            path='test.mysql.gz',
            show_stderr=False,
        ))


class DoPostgresqlBackupTestCase(PatchPipeCommandsTestCase):
    def test_it_makes_correct_calls_to_processes_api(self):
        do_postgresql_backup('test.pgsql.gz', DB_CONFIG)

        self.assertPipeCommandsToFileCallsEqual(call(
            [
                [
                    'pg_dump',
                    '--clean',
                    '--username=test_user',
                    '--host=test_host',
                    '--port=12345',
                    'test_db',
                ],
                ['gzip'],
            ],
            path='test.pgsql.gz',
            extra_env={'PGPASSWORD': 'test_password'},
            show_stderr=False,
        ))


class DoSqliteBackupTestCase(PatchPipeCommandsTestCase):
    def test_it_makes_correct_calls_to_processes_api(self):
        do_sqlite_backup('test.sqlite.gz', DB_CONFIG)

        self.assertPipeCommandsToFileCallsEqual(call(
            [['cat', 'test_db'], ['gzip']],
            path='test.sqlite.gz',
            show_stderr=False,
        ))


class DoMysqlRestoreTestCase(PatchPipeCommandsTestCase):
    def test_it_makes_correct_calls_to_processes_api(self):
        do_mysql_restore(backup_file='test.mysql.gz', db_config=DB_CONFIG)

        self.assertPipeCommandsCallsEqual(call(
            [
                ['cat', 'test.mysql.gz'],
                ['gunzip'],
                [
                    'mysql',
                    '--user=test_user',
                    '--password=test_password',
                    '--host=test_host',
                    '--port=12345',
                    'test_db',
                ],
            ],
            show_stderr=False,
            show_last_stdout=False,
        ))

    def test_it_drops_tables_before_restoring_if_specified(self):
        do_mysql_restore(backup_file='test.mysql.gz', db_config=DB_CONFIG, drop_tables=True)

        self.assertPipeCommandsCallsEqual(
            call(
                [
                    [
                        'mysqldump',
                        '--user=test_user',
                        '--password=test_password',
                        '--host=test_host',
                        '--port=12345',
                        'test_db',
                        '--no-data',
                    ],
                    ['grep', '^DROP'],
                    [
                        'mysql',
                        '--user=test_user',
                        '--password=test_password',
                        '--host=test_host',
                        '--port=12345',
                        'test_db',
                    ],
                ],
                show_stderr=False,
                show_last_stdout=False,
            ),
            call(
                [
                    ['cat', 'test.mysql.gz'],
                    ['gunzip'],
                    [
                        'mysql',
                        '--user=test_user',
                        '--password=test_password',
                        '--host=test_host',
                        '--port=12345',
                        'test_db',
                    ],
                ],
                show_stderr=False,
                show_last_stdout=False,
            ),
        )


class DoPostgresqlRestoreTestCase(PatchPipeCommandsTestCase):
    def test_it_makes_correct_calls_to_processes_api(self):
        do_postgresql_restore(backup_file='test.pgsql.gz', db_config=DB_CONFIG)

        self.assertPipeCommandsCallsEqual(call(
            [
                ['cat', 'test.pgsql.gz'],
                ['gunzip'],
                [
                    'psql',
                    '--username=test_user',
                    '--host=test_host',
                    '--port=12345',
                    'test_db',
                ],
            ],
            extra_env={'PGPASSWORD': 'test_password'},
            show_stderr=False,
            show_last_stdout=False,
        ))

    def test_it_drops_tables_before_restoring_if_specified(self):
        do_postgresql_restore(backup_file='test.pgsql.gz', db_config=DB_CONFIG, drop_tables=True)

        self.assertPipeCommandsCallsEqual(
            call(
                [
                    [
                        'psql',
                        '--username=test_user',
                        '--host=test_host',
                        '--port=12345',
                        'test_db',
                        '-t',
                        '-c',
                        PG_DROP_SQL,
                    ],
                    [
                        'psql',
                        '--username=test_user',
                        '--host=test_host',
                        '--port=12345',
                        'test_db',
                    ],
                ],
                extra_env={'PGPASSWORD': 'test_password'},
                show_stderr=False,
                show_last_stdout=False,
            ),
            call(
                [
                    ['cat', 'test.pgsql.gz'],
                    ['gunzip'],
                    [
                        'psql',
                        '--username=test_user',
                        '--host=test_host',
                        '--port=12345',
                        'test_db',
                    ],
                ],
                extra_env={'PGPASSWORD': 'test_password'},
                show_stderr=False,
                show_last_stdout=False,
            ),
        )


class DoSqliteRestoreTestCase(PatchPipeCommandsTestCase):
    def test_it_makes_correct_calls_to_processes_api(self):
        do_sqlite_restore(backup_file='test.sqlite.gz', db_config=DB_CONFIG)

        self.assertPipeCommandsToFileCallsEqual(call(
            [['cat', 'test.sqlite.gz'], ['gunzip']],
            path='test_db',
            show_stderr=False,
        ))



from subprocess import CalledProcessError
import os
import unittest

try:
    from collections import OrderedDict
except ImportError:
    # This should only happen in Python 2.6
    # SortedDict is deprecated in Django 1.7 and will be removed in Django 1.9
    from django.utils.datastructures import SortedDict as OrderedDict

from backupdb.utils.processes import (
    extend_env,
    get_env_str,
    pipe_commands,
    pipe_commands_to_file,
)

from .utils import FileSystemScratchTestCase


class ExtendEnvTestCase(unittest.TestCase):
    def test_extend_env_creates_a_copy_of_the_current_env(self):
        env = extend_env({'BACKUPDB_TEST_ENV_SETTING': 1234})
        self.assertFalse(env is os.environ)

    def test_extend_env_adds_keys_to_a_copy_of_the_current_env(self):
        env = extend_env({
            'BACKUPDB_TEST_ENV_SETTING_1': 1234,
            'BACKUPDB_TEST_ENV_SETTING_2': 1234,
        })

        orig_keys = set(os.environ.keys())
        curr_keys = set(env.keys())
        diff_keys = curr_keys - orig_keys

        self.assertEqual(diff_keys, set([
            'BACKUPDB_TEST_ENV_SETTING_1',
            'BACKUPDB_TEST_ENV_SETTING_2',
        ]))


class GetEnvStrTestCase(unittest.TestCase):
    def test_get_env_str_works_for_empty_dicts(self):
        self.assertEqual(get_env_str({}), '')

    def test_get_env_str_works_for_non_empty_dicts(self):
        self.assertEqual(
            get_env_str({'VAR_1': 1234}),
            "VAR_1='1234'",
        )
        self.assertEqual(
            get_env_str(OrderedDict([
                ('VAR_1', 1234),
                ('VAR_2', 'arst'),
            ])),
            "VAR_1='1234' VAR_2='arst'",
        )
        self.assertEqual(
            get_env_str(OrderedDict([
                ('VAR_1', 1234),
                ('VAR_2', 'arst'),
                ('VAR_3', 'zxcv'),
            ])),
            "VAR_1='1234' VAR_2='arst' VAR_3='zxcv'",
        )


class PipeCommandsTestCase(FileSystemScratchTestCase):
    def test_it_pipes_a_list_of_commands_into_each_other(self):
        pipe_commands([
            ['echo', r"""
import sys
for i in range(4):
    sys.stdout.write('spam\n')
"""],
            ['python'],
            ['tee', self.get_path('pipe_commands.out')],
        ])

        self.assertFileExists('pipe_commands.out')
        self.assertFileHasContent(
            'pipe_commands.out',
            'spam\nspam\nspam\nspam\n',
        )

    def test_it_works_when_large_amounts_of_data_are_being_piped(self):
        pipe_commands([
            ['echo', r"""
import sys
for i in range(400000):
    sys.stdout.write('spam\n')
"""],
            ['python'],
            ['tee', self.get_path('pipe_commands.out')],
        ])

        self.assertFileExists('pipe_commands.out')
        self.assertFileHasLength('pipe_commands.out', 2000000)
        self.assertInFile('pipe_commands.out', 'spam\nspam\nspam\n')

    def test_it_allows_you_to_specify_extra_environment_variables(self):
        pipe_commands([
            ['echo', """
import os
import sys
sys.stdout.write(os.environ['TEST_VAR'])
"""],
            ['python'],
            ['tee', self.get_path('pipe_commands.out')],
        ], extra_env={'TEST_VAR': 'spam'})

        self.assertFileExists('pipe_commands.out')
        self.assertFileHasContent('pipe_commands.out', 'spam')

    def test_it_correctly_raises_a_called_process_error_when_necessary(self):
        self.assertRaises(CalledProcessError, pipe_commands, [['false'], ['true']])


class PipeCommandsToFileTestCase(FileSystemScratchTestCase):
    def test_it_pipes_a_list_of_commands_into_each_other_and_then_into_a_file(self):
        pipe_commands_to_file([
            ['echo', r"""
import sys
for i in range(4):
    sys.stdout.write('spam\n')
"""],
            ['python'],
        ], self.get_path('pipe_commands.out'))

        self.assertFileExists('pipe_commands.out')
        self.assertFileHasContent(
            'pipe_commands.out',
            'spam\nspam\nspam\nspam\n',
        )

    def test_it_works_when_large_amounts_of_data_are_being_piped(self):
        pipe_commands_to_file([
            ['echo', r"""
import sys
for i in range(400000):
    sys.stdout.write('spam\n')
"""],
            ['python'],
        ], self.get_path('pipe_commands.out'))

        self.assertFileExists('pipe_commands.out')
        self.assertFileHasLength('pipe_commands.out', 2000000)
        self.assertInFile('pipe_commands.out', 'spam\nspam\nspam\n')

    def test_it_allows_you_to_specify_extra_environment_variables(self):
        pipe_commands_to_file([
            ['echo', """
import os
import sys
sys.stdout.write(os.environ['TEST_VAR'])
"""],
            ['python'],
        ], self.get_path('pipe_commands.out'), extra_env={'TEST_VAR': 'spam'})

        self.assertFileExists('pipe_commands.out')
        self.assertFileHasContent('pipe_commands.out', 'spam')

    def test_it_correctly_raises_a_called_process_error_when_necessary(self):
        self.assertRaises(
            CalledProcessError,
            pipe_commands_to_file,
            [['false'], ['true']],
            self.get_path('pipe_commands.out'),
        )



#!/usr/bin/env python
import subprocess
import sys
import os

from setuptools import setup


version = (0, 6, 0, 'final')

current_path = os.path.dirname(__file__)

sys.path.append(current_path)
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'test_settings')


def get_version():
    number = '.'.join(map(str, version[:3]))
    stage = version[3]
    if stage == 'final':
        return number
    elif stage == 'alpha':
        process = subprocess.Popen('git rev-parse HEAD'.split(), stdout=subprocess.PIPE)
        stdout, stderr = process.communicate()
        return number + '-' + stdout.decode('utf-8').strip()[:8]


def get_readme():
    with open(os.path.join(current_path, 'README.rst'), 'r') as f:
        return f.read()

setup(
    name='django-backupdb',
    version=get_version(),
    description='Management commands for backing up and restoring databases in Django.',
    long_description=get_readme(),
    author='Fusionbox programmers',
    author_email='programmers@fusionbox.com',
    keywords='django database backup',
    url='https://github.com/fusionbox/django-backupdb',
    packages=['backupdb', 'backupdb.utils', 'backupdb.management',
              'backupdb.management.commands', 'backupdb.tests',
              'backupdb.tests.app'],
    platforms='any',
    license='Fusionbox',
    test_suite='backupdb.tests.all_tests',
    tests_require=[
        'mock>=1.0.1',
        'dj_database_url==0.3.0',
    ],
    install_requires=[
        'Django>=1.4',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
    ],
)



import atexit
import dj_database_url
import os
import shutil
import tempfile

DEBUG = True

DATABASE = {
    'default': dj_database_url.config(default='sqlite://:memory:'),
}

INSTALLED_APPS = (
    'backupdb',
    'backupdb.tests.app',
)

SECRET_KEY = 'this is a secret!'

BACKUPDB_DIRECTORY = tempfile.mkdtemp()

# Cleanup after itself
atexit.register(shutil.rmtree, BACKUPDB_DIRECTORY)



import logging
import os
import shlex

from django.core.management.base import BaseCommand

from .exceptions import RestoreError
from .processes import pipe_commands, pipe_commands_to_file


PG_DROP_SQL = """SELECT 'DROP TABLE IF EXISTS "' || tablename || '" CASCADE;' FROM pg_tables WHERE schemaname = 'public';"""


class BaseBackupDbCommand(BaseCommand):
    can_import_settings = True

    LOG_LEVELS = {
        0: logging.ERROR,
        1: logging.INFO,
        2: logging.DEBUG,
        3: logging.DEBUG,
    }
    LOG_FORMAT = '%(asctime)s - %(levelname)-8s: %(message)s'

    def _setup_logging(self, level):
        level = int(level)
        logging.basicConfig(format=self.LOG_FORMAT, level=self.LOG_LEVELS[level])

    def handle(self, *args, **options):
        self._setup_logging(options['verbosity'])


def apply_arg_values(arg_values):
    """
    Apply argument to values.

    l = [('--name={0}', 'name'),
         ('--password={0}', 'password'),
         ('--level={0}', ''),
         ('--last={0}', None)]
    assert apply_arg_values(l) == ['--name=name', '--password=password']
    """
    return [a.format(v) for a, v in arg_values if v]


def require_backup_exists(func):
    """
    Requires that the file referred to by `backup_file` exists in the file
    system before running the decorated function.
    """
    def new_func(*args, **kwargs):
        backup_file = kwargs['backup_file']
        if not os.path.exists(backup_file):
            raise RestoreError("Could not find file '{0}'".format(backup_file))
        return func(*args, **kwargs)
    return new_func


def get_mysql_args(db_config):
    """
    Returns an array of argument values that will be passed to a `mysql` or
    `mysqldump` process when it is started based on the given database
    configuration.
    """
    db = db_config['NAME']

    mapping = [('--user={0}', db_config.get('USER')),
               ('--password={0}', db_config.get('PASSWORD')),
               ('--host={0}', db_config.get('HOST')),
               ('--port={0}', db_config.get('PORT'))]
    args = apply_arg_values(mapping)
    args.append(db)

    return args


def get_postgresql_args(db_config, extra_args=None):
    """
    Returns an array of argument values that will be passed to a `psql` or
    `pg_dump` process when it is started based on the given database
    configuration.
    """
    db = db_config['NAME']

    mapping = [('--username={0}', db_config.get('USER')),
               ('--host={0}', db_config.get('HOST')),
               ('--port={0}', db_config.get('PORT'))]
    args = apply_arg_values(mapping)

    if extra_args is not None:
        args.extend(shlex.split(extra_args))
    args.append(db)

    return args


def get_postgresql_env(db_config):
    """
    Returns a dict containing extra environment variable values that will be
    added to the environment of the `psql` or `pg_dump` process when it is
    started based on the given database configuration.
    """
    password = db_config.get('PASSWORD')
    return {'PGPASSWORD': password} if password else None


def do_mysql_backup(backup_file, db_config, show_output=False):
    args = get_mysql_args(db_config)

    cmd = ['mysqldump'] + args
    pipe_commands_to_file([cmd, ['gzip']], path=backup_file, show_stderr=show_output)


def do_postgresql_backup(backup_file, db_config, pg_dump_options=None, show_output=False):
    env = get_postgresql_env(db_config)
    args = get_postgresql_args(db_config, pg_dump_options)

    cmd = ['pg_dump', '--clean'] + args
    pipe_commands_to_file([cmd, ['gzip']], path=backup_file, extra_env=env, show_stderr=show_output)


def do_sqlite_backup(backup_file, db_config, show_output=False):
    db_file = db_config['NAME']

    cmd = ['cat', db_file]
    pipe_commands_to_file([cmd, ['gzip']], path=backup_file, show_stderr=show_output)


@require_backup_exists
def do_mysql_restore(backup_file, db_config, drop_tables=False, show_output=False):
    args = get_mysql_args(db_config)
    mysql_cmd = ['mysql'] + args

    kwargs = {'show_stderr': show_output, 'show_last_stdout': show_output}

    if drop_tables:
        dump_cmd = ['mysqldump'] + args + ['--no-data']
        pipe_commands([dump_cmd, ['grep', '^DROP'], mysql_cmd], **kwargs)

    pipe_commands([['cat', backup_file], ['gunzip'], mysql_cmd], **kwargs)


@require_backup_exists
def do_postgresql_restore(backup_file, db_config, drop_tables=False, show_output=False):
    env = get_postgresql_env(db_config)
    args = get_postgresql_args(db_config)
    psql_cmd = ['psql'] + args

    kwargs = {'extra_env': env, 'show_stderr': show_output, 'show_last_stdout': show_output}

    if drop_tables:
        gen_drop_sql_cmd = psql_cmd + ['-t', '-c', PG_DROP_SQL]
        pipe_commands([gen_drop_sql_cmd, psql_cmd], **kwargs)

    pipe_commands([['cat', backup_file], ['gunzip'], psql_cmd], **kwargs)


@require_backup_exists
def do_sqlite_restore(backup_file, db_config, drop_tables=False, show_output=False):
    db_file = db_config['NAME']

    cmd = ['cat', backup_file]
    pipe_commands_to_file([cmd, ['gunzip']], path=db_file, show_stderr=show_output)



from backupdb.utils.exceptions import RestoreError
from backupdb.utils.files import get_latest_timestamped_file

from backupdb.tests.utils import FileSystemScratchTestCase


class GetLatestTimestampedFileTestCase(FileSystemScratchTestCase):
    def create_files(self, *args):
        for file in args:
            open(self.get_path(file), 'a').close()

    def test_it_returns_the_latest_timestamped_file_with_ext(self):
        self.create_files(
            'default-2013-05-02-1367553089.sqlite.gz',
            'default-2013-06-06-1370570260.sqlite.gz',
            'default-2013-06-06-1370580510.sqlite.gz',

            'default-2013-05-02-1367553089.mysql.gz',
            'default-2013-06-06-1370570260.mysql.gz',
            'default-2013-06-06-1370580510.mysql.gz',

            'default-2013-05-02-1367553089.pgsql.gz',
            'default-2013-06-06-1370570260.pgsql.gz',
            'default-2013-06-06-1370580510.pgsql.gz',
        )

        sqlite_file = get_latest_timestamped_file('sqlite', dir=self.SCRATCH_DIR)
        mysql_file = get_latest_timestamped_file('mysql', dir=self.SCRATCH_DIR)
        pgsql_file = get_latest_timestamped_file('pgsql', dir=self.SCRATCH_DIR)

        self.assertEqual(sqlite_file, self.get_path('default-2013-06-06-1370580510.sqlite.gz'))
        self.assertEqual(mysql_file, self.get_path('default-2013-06-06-1370580510.mysql.gz'))
        self.assertEqual(pgsql_file, self.get_path('default-2013-06-06-1370580510.pgsql.gz'))

    def test_it_raises_an_exception_when_no_files_found(self):
        self.assertRaises(RestoreError, get_latest_timestamped_file, '', dir=self.SCRATCH_DIR)
        self.assertRaises(RestoreError, get_latest_timestamped_file, 'mysql', dir=self.SCRATCH_DIR)
